# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    PermissionPolicyResponse,
    PermissionPolicyAddToUserResponse,
    PermissionPolicyAddToUserGroupResponse,
    PermissionPolicyRemoveFromUserResponse,
    PermissionPolicyAddToServiceUserResponse,
    PermissionPolicyAddToSupportUserResponse,
    PermissionPolicyRemoveFromUserGroupResponse,
    PermissionPolicyRemoveFromServiceUserResponse,
    PermissionPolicyRemoveFromSupportUserResponse,
)
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPermissionPolicies:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
            version=0,
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.retrieve(
            id="id",
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.permission_policies.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
            version=0,
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.permission_policies.with_raw_response.update(
                id="",
                name="x",
                permission_policy=[
                    {
                        "action": ["ALL"],
                        "effect": "ALLOW",
                        "resource": ["string"],
                    }
                ],
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.list()
        assert_matches_type(SyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.list(
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(SyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(SyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.delete(
            id="id",
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.permission_policies.with_raw_response.delete(
                id="",
            )

    @parametrize
    def test_method_add_to_service_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_add_to_service_user_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_add_to_service_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_add_to_service_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_add_to_service_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.add_to_service_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    def test_method_add_to_support_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        )
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_add_to_support_user_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_support_user(
            permission_policy_id="permissionPolicyId",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_add_to_support_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_add_to_support_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_add_to_support_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.add_to_support_user(
                permission_policy_id="",
            )

    @parametrize
    def test_method_add_to_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_add_to_user_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_add_to_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_add_to_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_add_to_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.add_to_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    def test_method_add_to_user_group(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_add_to_user_group_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_add_to_user_group(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_add_to_user_group(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_add_to_user_group(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.add_to_user_group(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    def test_method_remove_from_service_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_remove_from_service_user_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_remove_from_service_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_remove_from_service_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_remove_from_service_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.remove_from_service_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    def test_method_remove_from_support_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        )
        assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_remove_from_support_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_remove_from_support_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_remove_from_support_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.remove_from_support_user(
                permission_policy_id="",
            )

    @parametrize
    def test_method_remove_from_user(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_remove_from_user_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_remove_from_user(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_remove_from_user(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_remove_from_user(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.remove_from_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    def test_method_remove_from_user_group(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_method_remove_from_user_group_with_all_params(self, client: M3ter) -> None:
        permission_policy = client.permission_policies.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_raw_response_remove_from_user_group(self, client: M3ter) -> None:
        response = client.permission_policies.with_raw_response.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = response.parse()
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    def test_streaming_response_remove_from_user_group(self, client: M3ter) -> None:
        with client.permission_policies.with_streaming_response.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = response.parse()
            assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_remove_from_user_group(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            client.permission_policies.with_raw_response.remove_from_user_group(
                permission_policy_id="",
                principal_id="x",
            )


class TestAsyncPermissionPolicies:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
            version=0,
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.create(
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.retrieve(
            id="id",
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.permission_policies.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
            version=0,
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.update(
            id="id",
            name="x",
            permission_policy=[
                {
                    "action": ["ALL"],
                    "effect": "ALLOW",
                    "resource": ["string"],
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.permission_policies.with_raw_response.update(
                id="",
                name="x",
                permission_policy=[
                    {
                        "action": ["ALL"],
                        "effect": "ALLOW",
                        "resource": ["string"],
                    }
                ],
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.list()
        assert_matches_type(AsyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.list(
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(AsyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(AsyncCursor[PermissionPolicyResponse], permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.delete(
            id="id",
        )
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.permission_policies.with_raw_response.delete(
                id="",
            )

    @parametrize
    async def test_method_add_to_service_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_add_to_service_user_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_add_to_service_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_add_to_service_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.add_to_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyAddToServiceUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_add_to_service_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.add_to_service_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    async def test_method_add_to_support_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        )
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_add_to_support_user_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_support_user(
            permission_policy_id="permissionPolicyId",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_add_to_support_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_add_to_support_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.add_to_support_user(
            permission_policy_id="permissionPolicyId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyAddToSupportUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_add_to_support_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.add_to_support_user(
                permission_policy_id="",
            )

    @parametrize
    async def test_method_add_to_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_add_to_user_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_add_to_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_add_to_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.add_to_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyAddToUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_add_to_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.add_to_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    async def test_method_add_to_user_group(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_add_to_user_group_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_add_to_user_group(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_add_to_user_group(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.add_to_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyAddToUserGroupResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_add_to_user_group(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.add_to_user_group(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    async def test_method_remove_from_service_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_remove_from_service_user_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_remove_from_service_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_remove_from_service_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.remove_from_service_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyRemoveFromServiceUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_remove_from_service_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.remove_from_service_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    async def test_method_remove_from_support_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        )
        assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_remove_from_support_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_remove_from_support_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.remove_from_support_user(
            permission_policy_id="permissionPolicyId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyRemoveFromSupportUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_remove_from_support_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.remove_from_support_user(
                permission_policy_id="",
            )

    @parametrize
    async def test_method_remove_from_user(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_remove_from_user_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_remove_from_user(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_remove_from_user(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.remove_from_user(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyRemoveFromUserResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_remove_from_user(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.remove_from_user(
                permission_policy_id="",
                principal_id="x",
            )

    @parametrize
    async def test_method_remove_from_user_group(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_method_remove_from_user_group_with_all_params(self, async_client: AsyncM3ter) -> None:
        permission_policy = await async_client.permission_policies.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
            version=0,
        )
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_raw_response_remove_from_user_group(self, async_client: AsyncM3ter) -> None:
        response = await async_client.permission_policies.with_raw_response.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission_policy = await response.parse()
        assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

    @parametrize
    async def test_streaming_response_remove_from_user_group(self, async_client: AsyncM3ter) -> None:
        async with async_client.permission_policies.with_streaming_response.remove_from_user_group(
            permission_policy_id="permissionPolicyId",
            principal_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission_policy = await response.parse()
            assert_matches_type(PermissionPolicyRemoveFromUserGroupResponse, permission_policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_remove_from_user_group(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_policy_id` but received ''"):
            await async_client.permission_policies.with_raw_response.remove_from_user_group(
                permission_policy_id="",
                principal_id="x",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.statements import (
    statement_definition_list_params,
    statement_definition_create_params,
    statement_definition_update_params,
)
from ...types.statement_definition_response import StatementDefinitionResponse

__all__ = ["StatementDefinitionsResource", "AsyncStatementDefinitionsResource"]


class StatementDefinitionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StatementDefinitionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StatementDefinitionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatementDefinitionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return StatementDefinitionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"],
        dimensions: Iterable[statement_definition_create_params.Dimension] | Omit = omit,
        generate_slim_statements: bool | Omit = omit,
        include_price_per_unit: bool | Omit = omit,
        measures: Iterable[statement_definition_create_params.Measure] | Omit = omit,
        name: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Create a new StatementDefinition.

        This endpoint creates a new StatementDefinition within the specified
        Organization. The details of the StatementDefinition are provided in the request
        body.

        Args:
          aggregation_frequency: This specifies how often the Statement should aggregate data.

          dimensions: An array of objects, each representing a Dimension data field from a Meter _(for
              Meters that have Dimensions setup)_.

          generate_slim_statements

          include_price_per_unit: A Boolean indicating whether to include the price per unit in the Statement.

              - TRUE - includes the price per unit.
              - FALSE - excludes the price per unit.

          measures: An array of objects, each representing a Measure data field from a Meter.

          name: Descriptive name for the StatementDefinition providing context and information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/statementdefinitions",
            body=maybe_transform(
                {
                    "aggregation_frequency": aggregation_frequency,
                    "dimensions": dimensions,
                    "generate_slim_statements": generate_slim_statements,
                    "include_price_per_unit": include_price_per_unit,
                    "measures": measures,
                    "name": name,
                    "version": version,
                },
                statement_definition_create_params.StatementDefinitionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Retrieve a StatementDefinition with the given UUID.

        Retrieves the details of a specific StatementDefinition for the specified
        Organization, using its unique identifier (UUID). This endpoint is useful when
        you want to retrieve the complete details of a single StatementDefinition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"],
        dimensions: Iterable[statement_definition_update_params.Dimension] | Omit = omit,
        generate_slim_statements: bool | Omit = omit,
        include_price_per_unit: bool | Omit = omit,
        measures: Iterable[statement_definition_update_params.Measure] | Omit = omit,
        name: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Update StatementDefinition for the given UUID.

        Update the details of a specific StatementDefinition for the specified
        Organization, using its unique identifier (UUID). The updated details for the
        StatementDefinition should be sent in the request body.

        Args:
          aggregation_frequency: This specifies how often the Statement should aggregate data.

          dimensions: An array of objects, each representing a Dimension data field from a Meter _(for
              Meters that have Dimensions setup)_.

          generate_slim_statements

          include_price_per_unit: A Boolean indicating whether to include the price per unit in the Statement.

              - TRUE - includes the price per unit.
              - FALSE - excludes the price per unit.

          measures: An array of objects, each representing a Measure data field from a Meter.

          name: Descriptive name for the StatementDefinition providing context and information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            body=maybe_transform(
                {
                    "aggregation_frequency": aggregation_frequency,
                    "dimensions": dimensions,
                    "generate_slim_statements": generate_slim_statements,
                    "include_price_per_unit": include_price_per_unit,
                    "measures": measures,
                    "name": name,
                    "version": version,
                },
                statement_definition_update_params.StatementDefinitionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[StatementDefinitionResponse]:
        """
        Retrieve a list of StatementDefinitions.

        This endpoint retrieves a list of all the StatementDefinitions within a
        specified Organization. The list can be paginated for easier management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              StatementDefinitions in a paginated list.

          page_size: Specifies the maximum number of StatementDefinitions to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/statementdefinitions",
            page=SyncCursor[StatementDefinitionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    statement_definition_list_params.StatementDefinitionListParams,
                ),
            ),
            model=StatementDefinitionResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Delete a StatementDefinition with the given UUID.

        This endpoint deletes a specific StatementDefinition within a specified
        Organization, using the StatementDefinition UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )


class AsyncStatementDefinitionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStatementDefinitionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatementDefinitionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatementDefinitionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncStatementDefinitionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"],
        dimensions: Iterable[statement_definition_create_params.Dimension] | Omit = omit,
        generate_slim_statements: bool | Omit = omit,
        include_price_per_unit: bool | Omit = omit,
        measures: Iterable[statement_definition_create_params.Measure] | Omit = omit,
        name: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Create a new StatementDefinition.

        This endpoint creates a new StatementDefinition within the specified
        Organization. The details of the StatementDefinition are provided in the request
        body.

        Args:
          aggregation_frequency: This specifies how often the Statement should aggregate data.

          dimensions: An array of objects, each representing a Dimension data field from a Meter _(for
              Meters that have Dimensions setup)_.

          generate_slim_statements

          include_price_per_unit: A Boolean indicating whether to include the price per unit in the Statement.

              - TRUE - includes the price per unit.
              - FALSE - excludes the price per unit.

          measures: An array of objects, each representing a Measure data field from a Meter.

          name: Descriptive name for the StatementDefinition providing context and information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/statementdefinitions",
            body=await async_maybe_transform(
                {
                    "aggregation_frequency": aggregation_frequency,
                    "dimensions": dimensions,
                    "generate_slim_statements": generate_slim_statements,
                    "include_price_per_unit": include_price_per_unit,
                    "measures": measures,
                    "name": name,
                    "version": version,
                },
                statement_definition_create_params.StatementDefinitionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Retrieve a StatementDefinition with the given UUID.

        Retrieves the details of a specific StatementDefinition for the specified
        Organization, using its unique identifier (UUID). This endpoint is useful when
        you want to retrieve the complete details of a single StatementDefinition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["DAY", "WEEK", "MONTH", "QUARTER", "YEAR", "WHOLE_PERIOD"],
        dimensions: Iterable[statement_definition_update_params.Dimension] | Omit = omit,
        generate_slim_statements: bool | Omit = omit,
        include_price_per_unit: bool | Omit = omit,
        measures: Iterable[statement_definition_update_params.Measure] | Omit = omit,
        name: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Update StatementDefinition for the given UUID.

        Update the details of a specific StatementDefinition for the specified
        Organization, using its unique identifier (UUID). The updated details for the
        StatementDefinition should be sent in the request body.

        Args:
          aggregation_frequency: This specifies how often the Statement should aggregate data.

          dimensions: An array of objects, each representing a Dimension data field from a Meter _(for
              Meters that have Dimensions setup)_.

          generate_slim_statements

          include_price_per_unit: A Boolean indicating whether to include the price per unit in the Statement.

              - TRUE - includes the price per unit.
              - FALSE - excludes the price per unit.

          measures: An array of objects, each representing a Measure data field from a Meter.

          name: Descriptive name for the StatementDefinition providing context and information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            body=await async_maybe_transform(
                {
                    "aggregation_frequency": aggregation_frequency,
                    "dimensions": dimensions,
                    "generate_slim_statements": generate_slim_statements,
                    "include_price_per_unit": include_price_per_unit,
                    "measures": measures,
                    "name": name,
                    "version": version,
                },
                statement_definition_update_params.StatementDefinitionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[StatementDefinitionResponse, AsyncCursor[StatementDefinitionResponse]]:
        """
        Retrieve a list of StatementDefinitions.

        This endpoint retrieves a list of all the StatementDefinitions within a
        specified Organization. The list can be paginated for easier management.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              StatementDefinitions in a paginated list.

          page_size: Specifies the maximum number of StatementDefinitions to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/statementdefinitions",
            page=AsyncCursor[StatementDefinitionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    statement_definition_list_params.StatementDefinitionListParams,
                ),
            ),
            model=StatementDefinitionResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementDefinitionResponse:
        """
        Delete a StatementDefinition with the given UUID.

        This endpoint deletes a specific StatementDefinition within a specified
        Organization, using the StatementDefinition UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/statementdefinitions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementDefinitionResponse,
        )


class StatementDefinitionsResourceWithRawResponse:
    def __init__(self, statement_definitions: StatementDefinitionsResource) -> None:
        self._statement_definitions = statement_definitions

        self.create = to_raw_response_wrapper(
            statement_definitions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            statement_definitions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            statement_definitions.update,
        )
        self.list = to_raw_response_wrapper(
            statement_definitions.list,
        )
        self.delete = to_raw_response_wrapper(
            statement_definitions.delete,
        )


class AsyncStatementDefinitionsResourceWithRawResponse:
    def __init__(self, statement_definitions: AsyncStatementDefinitionsResource) -> None:
        self._statement_definitions = statement_definitions

        self.create = async_to_raw_response_wrapper(
            statement_definitions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            statement_definitions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            statement_definitions.update,
        )
        self.list = async_to_raw_response_wrapper(
            statement_definitions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            statement_definitions.delete,
        )


class StatementDefinitionsResourceWithStreamingResponse:
    def __init__(self, statement_definitions: StatementDefinitionsResource) -> None:
        self._statement_definitions = statement_definitions

        self.create = to_streamed_response_wrapper(
            statement_definitions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            statement_definitions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            statement_definitions.update,
        )
        self.list = to_streamed_response_wrapper(
            statement_definitions.list,
        )
        self.delete = to_streamed_response_wrapper(
            statement_definitions.delete,
        )


class AsyncStatementDefinitionsResourceWithStreamingResponse:
    def __init__(self, statement_definitions: AsyncStatementDefinitionsResource) -> None:
        self._statement_definitions = statement_definitions

        self.create = async_to_streamed_response_wrapper(
            statement_definitions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            statement_definitions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            statement_definitions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            statement_definitions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            statement_definitions.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balances import transaction_list_params, transaction_create_params
from ...types.balances.transaction_response import TransactionResponse
from ...types.balances.transaction_summary_response import TransactionSummaryResponse

__all__ = ["TransactionsResource", "AsyncTransactionsResource"]


class TransactionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return TransactionsResourceWithStreamingResponse(self)

    def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        applied_date: Union[str, datetime] | Omit = omit,
        currency_paid: str | Omit = omit,
        description: str | Omit = omit,
        paid: float | Omit = omit,
        transaction_date: Union[str, datetime] | Omit = omit,
        transaction_type_id: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionResponse:
        """Add a Transaction to a Balance.

        This endpoint allows you to create a new
        Transaction amount for a Balance. This amount then becomes available at billing
        for draw-down to cover charges due. The Transaction details should be provided
        in the request body.

        Before you can add a Transaction amount, you must first set up Transaction Types
        at the Organization Level - see the
        [Transaction Type](https://www.m3ter.com/docs/api#tag/TransactionType) section
        in this API Reference for more details. You can then use this call to add an
        instance of a Transaction Type to a Balance.

        **Note:** If you have a customer whose payment is in a different currency to the
        Balance currency, you can use the `paid` and `paidCurrency` request parameters
        to record the amount paid and alternative currency respectively. For example,
        you might add a Transaction amount of 200 USD to a Balance on a customer Account
        where the customer actually paid you 50 units in virtual currency X.

        Args:
          amount: The financial value of the transaction.

          applied_date: The date _(in ISO 8601 format)_ when the Balance transaction was applied.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          description: A brief description explaining the purpose and context of the transaction.

          paid: The payment amount if the payment currency differs from the Balance currency.

          transaction_date: The date _(in ISO 8601 format)_ when the transaction occurred.

          transaction_type_id: The unique identifier (UUID) of the transaction type. This is obtained from the
              list of created Transaction Types within the Organization Configuration.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            body=maybe_transform(
                {
                    "amount": amount,
                    "applied_date": applied_date,
                    "currency_paid": currency_paid,
                    "description": description,
                    "paid": paid,
                    "transaction_date": transaction_date,
                    "transaction_type_id": transaction_type_id,
                    "version": version,
                },
                transaction_create_params.TransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        entity_id: Optional[str] | Omit = omit,
        entity_type: Optional[Literal["BILL", "COMMITMENT", "USER", "SERVICE_USER", "SCHEDULER"]] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        transaction_type_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[TransactionResponse]:
        """
        Retrieve all Transactions for a specific Balance.

        This endpoint returns a list of all Transactions associated with a specific
        Balance. You can paginate through the Transactions by using the `pageSize` and
        `nextToken` parameters.

        Args:
          entity_type

          next_token: `nextToken` for multi page retrievals. A token for retrieving the next page of
              transactions. You'll get this from the response to your request.

          page_size: The maximum number of transactions to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            page=SyncCursor[TransactionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "next_token": next_token,
                        "page_size": page_size,
                        "transaction_type_id": transaction_type_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            model=TransactionResponse,
        )

    def summary(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionSummaryResponse:
        """
        Retrieves the Balance Transactions Summary for a given Balance.

        The response contains useful recorded and calculated Transaction amounts created
        for a Balance during the time it is active for the Account, including amounts
        relevant to any rollover amount configured for a Balance:

        - `totalCreditAmount`. The sum of all credits amounts created for the Balance.
        - `totalDebitAmount`. The sum of all debit amounts created for the Balance.
        - `initialCreditAmount`. The initial credit amount created for the Balance.
        - `expiredBalanceAmount`. The amount of the Balance remaining at the time the
          Balance expires and which is not included in any configured Rollover amount.
          For example, suppose a Balance reaches its end date and $1000 credit remains
          unused. If the Balance is configured to rollover $800, then the
          `expiredBalanceAmount` is calculated as $1000 - $800 = $200.
        - `rolloverConsumed`. The sum of debits made against the configured rollover
          amount. Note that this amount is dynamic relative to when the API call is made
          until either the rollover end date is reached or the cap configured for the
          rollover amount is reached, after which it will be unchanged. If no rollover
          is configured for a Balance, then this is ignored.
        - `balanceConsumed`. The sum of debits made against the Balance. Note that this
          amount is dynamic relative to when the API call is made until either the
          Balance end date is reached or the available Balance amount reaches zero,
          after which it will be unchanged.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{balance_id}/transactions/summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionSummaryResponse,
        )


class AsyncTransactionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncTransactionsResourceWithStreamingResponse(self)

    async def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        applied_date: Union[str, datetime] | Omit = omit,
        currency_paid: str | Omit = omit,
        description: str | Omit = omit,
        paid: float | Omit = omit,
        transaction_date: Union[str, datetime] | Omit = omit,
        transaction_type_id: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionResponse:
        """Add a Transaction to a Balance.

        This endpoint allows you to create a new
        Transaction amount for a Balance. This amount then becomes available at billing
        for draw-down to cover charges due. The Transaction details should be provided
        in the request body.

        Before you can add a Transaction amount, you must first set up Transaction Types
        at the Organization Level - see the
        [Transaction Type](https://www.m3ter.com/docs/api#tag/TransactionType) section
        in this API Reference for more details. You can then use this call to add an
        instance of a Transaction Type to a Balance.

        **Note:** If you have a customer whose payment is in a different currency to the
        Balance currency, you can use the `paid` and `paidCurrency` request parameters
        to record the amount paid and alternative currency respectively. For example,
        you might add a Transaction amount of 200 USD to a Balance on a customer Account
        where the customer actually paid you 50 units in virtual currency X.

        Args:
          amount: The financial value of the transaction.

          applied_date: The date _(in ISO 8601 format)_ when the Balance transaction was applied.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          description: A brief description explaining the purpose and context of the transaction.

          paid: The payment amount if the payment currency differs from the Balance currency.

          transaction_date: The date _(in ISO 8601 format)_ when the transaction occurred.

          transaction_type_id: The unique identifier (UUID) of the transaction type. This is obtained from the
              list of created Transaction Types within the Organization Configuration.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "applied_date": applied_date,
                    "currency_paid": currency_paid,
                    "description": description,
                    "paid": paid,
                    "transaction_date": transaction_date,
                    "transaction_type_id": transaction_type_id,
                    "version": version,
                },
                transaction_create_params.TransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        entity_id: Optional[str] | Omit = omit,
        entity_type: Optional[Literal["BILL", "COMMITMENT", "USER", "SERVICE_USER", "SCHEDULER"]] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        transaction_type_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TransactionResponse, AsyncCursor[TransactionResponse]]:
        """
        Retrieve all Transactions for a specific Balance.

        This endpoint returns a list of all Transactions associated with a specific
        Balance. You can paginate through the Transactions by using the `pageSize` and
        `nextToken` parameters.

        Args:
          entity_type

          next_token: `nextToken` for multi page retrievals. A token for retrieving the next page of
              transactions. You'll get this from the response to your request.

          page_size: The maximum number of transactions to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            page=AsyncCursor[TransactionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "next_token": next_token,
                        "page_size": page_size,
                        "transaction_type_id": transaction_type_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            model=TransactionResponse,
        )

    async def summary(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionSummaryResponse:
        """
        Retrieves the Balance Transactions Summary for a given Balance.

        The response contains useful recorded and calculated Transaction amounts created
        for a Balance during the time it is active for the Account, including amounts
        relevant to any rollover amount configured for a Balance:

        - `totalCreditAmount`. The sum of all credits amounts created for the Balance.
        - `totalDebitAmount`. The sum of all debit amounts created for the Balance.
        - `initialCreditAmount`. The initial credit amount created for the Balance.
        - `expiredBalanceAmount`. The amount of the Balance remaining at the time the
          Balance expires and which is not included in any configured Rollover amount.
          For example, suppose a Balance reaches its end date and $1000 credit remains
          unused. If the Balance is configured to rollover $800, then the
          `expiredBalanceAmount` is calculated as $1000 - $800 = $200.
        - `rolloverConsumed`. The sum of debits made against the configured rollover
          amount. Note that this amount is dynamic relative to when the API call is made
          until either the rollover end date is reached or the cap configured for the
          rollover amount is reached, after which it will be unchanged. If no rollover
          is configured for a Balance, then this is ignored.
        - `balanceConsumed`. The sum of debits made against the Balance. Note that this
          amount is dynamic relative to when the API call is made until either the
          Balance end date is reached or the available Balance amount reaches zero,
          after which it will be unchanged.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{balance_id}/transactions/summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionSummaryResponse,
        )


class TransactionsResourceWithRawResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.create = to_raw_response_wrapper(
            transactions.create,
        )
        self.list = to_raw_response_wrapper(
            transactions.list,
        )
        self.summary = to_raw_response_wrapper(
            transactions.summary,
        )


class AsyncTransactionsResourceWithRawResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.create = async_to_raw_response_wrapper(
            transactions.create,
        )
        self.list = async_to_raw_response_wrapper(
            transactions.list,
        )
        self.summary = async_to_raw_response_wrapper(
            transactions.summary,
        )


class TransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.create = to_streamed_response_wrapper(
            transactions.create,
        )
        self.list = to_streamed_response_wrapper(
            transactions.list,
        )
        self.summary = to_streamed_response_wrapper(
            transactions.summary,
        )


class AsyncTransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.create = async_to_streamed_response_wrapper(
            transactions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            transactions.list,
        )
        self.summary = async_to_streamed_response_wrapper(
            transactions.summary,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DownloadURLResponse"]


class DownloadURLResponse(BaseModel):
    """It contains details for downloading a file"""

    url: Optional[str] = None
    """The presigned download URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import date, datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ChargeScheduleUpdateParams"]


class ChargeScheduleUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    balance_id: Required[Annotated[str, PropertyInfo(alias="balanceId")]]

    bill_frequency: Required[
        Annotated[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"], PropertyInfo(alias="billFrequency")]
    ]
    """Represents standard scheduling frequencies options for a job."""

    bill_frequency_interval: Required[Annotated[int, PropertyInfo(alias="billFrequencyInterval")]]
    """How often Bills are issued.

    For example, if billFrequency is `MONTHLY` and `billFrequencyInterval` is 3,
    Bills are issued every three months.
    """

    bill_in_advance: Required[Annotated[bool, PropertyInfo(alias="billInAdvance")]]
    """
    Used to specify how Charges created by the Balance Charge Schedule are billed -
    either in arrears or in advance:

    - Use `false` for billing in arrears.
    - Use `true` for billing in advance.
    """

    charge_description: Required[Annotated[str, PropertyInfo(alias="chargeDescription")]]
    """The description for Charges created by the Balance Charge Schedule.

    Used on Bills for Charge line items.
    """

    code: Required[str]
    """Unique short code for the Balance Charge Schedule."""

    currency: Required[str]
    """The currency of the Charges created by the Balance Charge Schedule."""

    name: Required[str]
    """The name of the Balance Charge Schedule."""

    service_period_end_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodEndDate", format="iso8601")]
    ]
    """
    The service period end date (_in ISO-8601 format_) of the Balance Charge
    Schedule.
    """

    service_period_start_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodStartDate", format="iso8601")]
    ]
    """
    The service period start date (_in ISO-8601 format)_ of the Balance Charge
    Schedule.
    """

    amount: float
    """The amount of each Charge created by the Balance Charge Schedule.

    Must be omitted if `units` and `unitPrice` are provided.
    """

    bill_epoch: Annotated[Union[str, date], PropertyInfo(alias="billEpoch", format="iso8601")]
    """
    Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
    created for Balance Charges created by the Schedule, and also acts as a
    reference for when in the Schedule period subsequent Bills are created for the
    defined `billFrequency`. If left blank, then the relevant Epoch date from your
    Organization's configuration will be used as the billing cycle date instead.
    """

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    unit_price: Annotated[float, PropertyInfo(alias="unitPrice")]
    """Unit price for Charge.

    Must be provided when `units` is used. Must be omitted when `amount` is used.
    """

    units: float
    """Number of units defined for the Charges created by the Schedule.

    Required when `unitPrice` is provided. Must be omitted when `amount` is used.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .statement_job_list_params import StatementJobListParams as StatementJobListParams
from .statement_job_create_params import StatementJobCreateParams as StatementJobCreateParams
from .statement_definition_list_params import StatementDefinitionListParams as StatementDefinitionListParams
from .statement_job_create_batch_params import StatementJobCreateBatchParams as StatementJobCreateBatchParams
from .statement_definition_create_params import StatementDefinitionCreateParams as StatementDefinitionCreateParams
from .statement_definition_update_params import StatementDefinitionUpdateParams as StatementDefinitionUpdateParams
from .statement_job_create_batch_response import StatementJobCreateBatchResponse as StatementJobCreateBatchResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ChargeUpdateResponse"]


class ChargeUpdateResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """The ID of the Account the Charge was created for."""

    accounting_product_id: Optional[str] = FieldInfo(alias="accountingProductId", default=None)
    """The Accounting Product ID assigned to the Charge."""

    amount: Optional[float] = None
    """The Charge amount.

    If `amount` has been defined, then `units` and `unitPrice` cannot be used.
    """

    bill_date: Optional[date] = FieldInfo(alias="billDate", default=None)
    """The date when the Charge will be added to a Bill."""

    bill_id: Optional[str] = FieldInfo(alias="billId", default=None)
    """The ID of the Bill created for this Charge."""

    code: Optional[str] = None
    """The unique short code of the Charge."""

    contract_id: Optional[str] = FieldInfo(alias="contractId", default=None)
    """The ID of a Contract on the Account that the Charge has been added to."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created the Charge."""

    currency: Optional[str] = None
    """Charge currency."""

    description: Optional[str] = None
    """The description added to the Bill line item for the Charge."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time (_in ISO-8601 format_) when the Charge was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time (_in ISO 8601 format_) when the Charge was last modified."""

    entity_id: Optional[str] = FieldInfo(alias="entityId", default=None)
    """The ID of the Charge linked entity.

    For example, the ID of an Account Balance if a Balance Charge.
    """

    entity_type: Optional[Literal["AD_HOC", "BALANCE"]] = FieldInfo(alias="entityType", default=None)
    """The entity type the Charge has been created for."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified the Charge."""

    line_item_type: Optional[Literal["BALANCE_FEE", "AD_HOC"]] = FieldInfo(alias="lineItemType", default=None)
    """The line item type used for billing a Charge."""

    name: Optional[str] = None
    """Name of the Charge. Added to the Bill line item description for Charge."""

    notes: Optional[str] = None
    """
    Information about the Charge for accounting purposes, such as the reason it was
    created. This information will not be added to the created Bill line item for
    the Charge.
    """

    schedule_id: Optional[str] = FieldInfo(alias="scheduleId", default=None)
    """The ID of the Balance Charge Schedule that created the Charge."""

    service_period_end_date: Optional[datetime] = FieldInfo(alias="servicePeriodEndDate", default=None)
    """The service period end date (_in ISO-8601 format_) for the Charge.

    **NOTE:** End date is exclusive.
    """

    service_period_start_date: Optional[datetime] = FieldInfo(alias="servicePeriodStartDate", default=None)
    """The service period start date (_in ISO-8601 format_) for the Charge ."""

    unit_price: Optional[float] = FieldInfo(alias="unitPrice", default=None)
    """Unit Price for the Charge. Provided together with `units`:

    - Null if the Charge was created with `amount` only.
    - If `units` and `unitPrice` are provided, `amount` cannot be used.
    """

    units: Optional[float] = None
    """Number of units of the Charge.

    Provided together with `unitPrice`. If `units` and `unitPrice` are provided,
    `amount` cannot be used.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["LookupTableRetrieveParams"]


class LookupTableRetrieveParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    additional: SequenceNotStr[str]
    """
    Comma separated list of additional non-default fields to be included in the
    response. For example,if you want to include the active Revision for the Lookup
    Tables returned, set `additional=activeRevision` in the query.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["DataExportDestinationGoogleCloudStorageRequestParam"]


class DataExportDestinationGoogleCloudStorageRequestParam(TypedDict, total=False):
    bucket_name: Required[Annotated[str, PropertyInfo(alias="bucketName")]]
    """The export destination bucket name."""

    pool_id: Required[Annotated[str, PropertyInfo(alias="poolId")]]
    """The export destination Web Identity Federation poolId."""

    project_number: Required[Annotated[str, PropertyInfo(alias="projectNumber")]]
    """The export destination GCP projectNumber."""

    provider_id: Required[Annotated[str, PropertyInfo(alias="providerId")]]
    """The export destination Web Identity Federation identity providerId."""

    destination_type: Annotated[Literal["GCS"], PropertyInfo(alias="destinationType")]
    """The type of destination to create. Possible values are: GCS"""

    partition_order: Annotated[Optional[Literal["TYPE_FIRST", "TIME_FIRST"]], PropertyInfo(alias="partitionOrder")]
    """
    Specify how you want the file path to be structured in your bucket destination -
    by Time first (Default) or Type first.

    Type is dependent on whether the Export is for Usage data or Operational data:

    - **Usage.** Type is `measurements`.
    - **Operational.** Type is one of the entities for which operational data
      exports are available, such as `account`, `commitment`, `meter`, and so on.

    Example for Usage Data Export using .CSV format:

    - Time first:
      `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    - Type first:
      `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    """

    prefix: str
    """The export destination prefix."""

    service_account_email: Annotated[str, PropertyInfo(alias="serviceAccountEmail")]
    """The export destination service account email."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..statement_job_response import StatementJobResponse

__all__ = ["StatementJobCreateBatchResponse"]

StatementJobCreateBatchResponse: TypeAlias = List[StatementJobResponse]
